#include <SPI.h>
#include <SD.h>
#include <Wire.h>
#include "RTClib.h"

// A simple data logger for the Arduino analog pins

// how many milliseconds between grabbing data and logging it. 1000 ms is once a second
#define LOG_INTERVAL  5000 // mills between entries (reduce to take more/faster data)

// how many milliseconds before writing the logged data permanently to disk
// set it to the LOG_INTERVAL to write each time (safest)
// set it to 10*LOG_INTERVAL to write all data every 10 datareads, you could lose up to
// the last 10 reads if power is lost but it uses less power and is much faster!
#define SYNC_INTERVAL 10000 // mills between calls to flush() - to write data to the card
uint32_t syncTime = 0; // time of last sync()

#define ECHO_TO_SERIAL   1 // echo data to serial port
#define WAIT_TO_START    0 // Wait for serial input in setup()
int LickDuration1 = 0;
int LickNumber1 = 0;
int LickDuration2 = 0;
int LickNumber2 = 0;
int LOG_INTERVAL_TIMER = 0;
unsigned long previousMillis = 0;

// the digital pins that connect to the LEDs
#define lick1 7  //Lick 1
#define lick2 8  //Lick 1
#define redLEDpin 13

// The analog pins that connect to the sensors
#define BANDGAPREF 14            // special indicator that we want to measure the bandgap

#define aref_voltage 3.3         // we tie 3.3V to ARef and measure it with a multimeter!
#define bandgap_voltage 1.1      // this is not super guaranteed but its not -too- off

RTC_DS1307 RTC; // define the Real Time Clock object

// for the data logging shield, we use digital pin 10 for the SD cs line
const int chipSelect = 10;

// the logging file
File logfile;

void error(char *str)
{
  Serial.print("error: ");
  Serial.println(str);
  while (1);
}

void setup(void)
{
  Serial.begin(9600);
  Serial.println();


#if WAIT_TO_START
  Serial.println("Type any character to start");
  while (!Serial.available());
#endif //WAIT_TO_START

  // initialize the SD card
  Serial.print("Initializing SD card...");
  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(10, OUTPUT);
  pinMode(7, INPUT);

  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    error("Card failed, or not present");
  }
  Serial.println("card initialized.");

  // create a new file
  char filename[] = "LOGGER00.CSV"; //To do, get this to be datestamp
  for (uint8_t i = 0; i < 100; i++) {
    filename[6] = i / 10 + '0';
    filename[7] = i % 10 + '0';
    if (! SD.exists(filename)) {
      // only open a new file if it doesn't exist
      logfile = SD.open(filename, FILE_WRITE);
      break;  // leave the loop!
    }
  }

  if (! logfile) {
    error("couldnt create file");
  }

  Serial.print("Logging to: ");
  Serial.println(filename);

  // connect to RTC
  Wire.begin();
  if (!RTC.begin()) {
    logfile.println("RTC failed");
#if ECHO_TO_SERIAL
    Serial.println("RTC failed");
#endif  //ECHO_TO_SERIAL
  }

  logfile.println("millis,stamp,datetime,LickNumber1,LickDuration1,LickNumber2,LickDuration2");
#if ECHO_TO_SERIAL
  Serial.println("millis,stamp,datetime,LickNumber1,LickDuration1,LickNumber2,LickDuration2");
#endif //ECHO_TO_SERIAL

  // If you want to set the aref to something other than 5v
  analogReference(EXTERNAL);
}

void loop(void)
{
  DateTime now;
  //Reset lick variables and timer
  LOG_INTERVAL_TIMER = 0;
  LickNumber1 = 0;
  LickDuration1 = 0;
  LickNumber2 = 0;
  LickDuration2 = 0;
  unsigned long currentMillis = millis();
  previousMillis = currentMillis;

  while (currentMillis < (previousMillis + LOG_INTERVAL)) {
    currentMillis = millis();
    
    if (digitalRead(lick1) == LOW) {
      LickDuration1 += 1;
      delay (1);
      if (digitalRead(lick1) == HIGH) {
        LickNumber1 += 1;
      }
    }
    if (digitalRead(lick2) == LOW) {
      LickDuration2 += 1;
      delay (1);
      if (digitalRead(lick2) == HIGH) {
        LickNumber2 += 1;
      }
    }
  }

  // log milliseconds since starting
  uint32_t m = millis();
  logfile.print(m);           // milliseconds since start
  logfile.print(", ");
#if ECHO_TO_SERIAL
  Serial.print(m);         // milliseconds since start
  Serial.print(", ");
#endif

  // fetch the time
  now = RTC.now();
  // log time
  logfile.print(now.unixtime()); // seconds since 1/1/1970
  logfile.print(", ");
  logfile.print('"');
  logfile.print(now.year(), DEC);
  logfile.print("/");
  logfile.print(now.month(), DEC);
  logfile.print("/");
  logfile.print(now.day(), DEC);
  logfile.print(" ");
  logfile.print(now.hour(), DEC);
  logfile.print(":");
  logfile.print(now.minute(), DEC);
  logfile.print(":");
  logfile.print(now.second(), DEC);
  logfile.print('"');
#if ECHO_TO_SERIAL
  Serial.print(now.unixtime()); // seconds since 1/1/1970
  Serial.print(", ");
  Serial.print('"');
  Serial.print(now.year(), DEC);
  Serial.print("/");
  Serial.print(now.month(), DEC);
  Serial.print("/");
  Serial.print(now.day(), DEC);
  Serial.print(" ");
  Serial.print(now.hour(), DEC);
  Serial.print(":");
  Serial.print(now.minute(), DEC);
  Serial.print(":");
  Serial.print(now.second(), DEC);
  Serial.print('"');
#endif //ECHO_TO_SERIAL
  logfile.print(", ");
  logfile.print(LickNumber1);
  logfile.print(", ");
  logfile.print(LickDuration1);
  logfile.print(", ");
  logfile.print(LickNumber2);
  logfile.print(", ");
  logfile.print(LickDuration2);

#if ECHO_TO_SERIAL
  Serial.print(", ");
  Serial.print(LickNumber1);
  Serial.print(", ");
  Serial.print(LickDuration1);
  Serial.print(", ");
  Serial.print(LickNumber2);
  Serial.print(", ");
  Serial.print(LickDuration2);
#endif //ECHO_TO_SERIAL

  logfile.println();  // this is going to next line
#if ECHO_TO_SERIAL
  Serial.println();
#endif // ECHO_TO_SERIAL

  // Now we write data to disk! Don't sync too often - requires 2048 bytes of I/O to SD card
  // which uses a bunch of power and takes time
  if ((millis() - syncTime) < SYNC_INTERVAL) return;
  syncTime = millis();

  // blink LED to show we are syncing data to the card & updating FAT!
  digitalWrite(redLEDpin, HIGH);
  logfile.flush();
  digitalWrite(redLEDpin, LOW);
}


